@@ -1,11 +1,11 @@
 """Module for FuzzySearcher class. Does fuzzy matching in spaCy Docs."""
 from itertools import chain
-from typing import Dict, List, Tuple, Union
+from typing import Callable, Dict, List, Tuple, Union
 import warnings
 
+from rapidfuzz import fuzz
 from spacy.tokens import Doc
 
-from .fuzzyconfig import FuzzyConfig
 from ..exceptions import FlexWarning
 
 
@@ -17,126 +17,41 @@ class FuzzySearcher:
     span in a Doc object, the n best fuzzy matched spans in a Doc,
     and fuzzy matching between any two given strings.
 
+    Fuzzy matching is currently provided by rapidfuzz and the searcher
+    contains provides access to all rapidfuzz matchers with default
+    settings.
+
     Attributes:
-        _config (FuzzyConfig): The fuzzy config used with the fuzzy searcher.
+        _fuzzy_funcs (Dict[str, Callable[[str, str], int]]):
+            Fuzzy matching functions accessible
+            by their given key name. All rapidfuzz matchers
+            with default settings are currently available:
+            "simple" = fuzz.ratio
+            "partial" = fuzz.partial_ratio
+            "token_set" = fuzz.token_set_ratio
+            "token_sort" = fuzz.token_sort_ratio
+            "partial_token_set" = fuzz.partial_token_set_ratio
+            "partial_token_sort" = fuzz.partial_token_sort_ratio
+            "quick" = fuzz.QRatio
+            "weighted" = fuzz.WRatio
+            "quick_lev" = fuzz.quick_lev_ratio
     """
 
-    def __init__(self, config: Union[str, FuzzyConfig] = "default") -> None:
-        """Initializes a fuzzy searcher with the given config.
+    def __init__(self) -> None:
+        """Initializes a fuzzy searcher with the given config."""
+        self._fuzzy_funcs: Dict[str, Callable[[str, str], int]] = {
+            "simple": fuzz.ratio,
+            "partial": fuzz.partial_ratio,
+            "token_set": fuzz.token_set_ratio,
+            "token_sort": fuzz.token_sort_ratio,
+            "partial_token_set": fuzz.partial_token_set_ratio,
+            "partial_token_sort": fuzz.partial_token_sort_ratio,
+            "quick": fuzz.QRatio,
+            "weighted": fuzz.WRatio,
+            "quick_lev": fuzz.quick_lev_ratio,
+        }
 
-        Args:
-            config: Provides predefind fuzzy matching functions.
-                Uses the default config if "default", an empty config if "empty",
-                or a custom config by passing a FuzzyConfig object.
-                Default is "default".
-
-        Raises:
-            TypeError: If config is not a FuzzyConfig object.
-        """
-        if config == "default":
-            self._config = FuzzyConfig(empty=False)
-        elif config == "empty":
-            self._config = FuzzyConfig(empty=True)
-        else:
-            if isinstance(config, FuzzyConfig):
-                self._config = config
-            else:
-                raise TypeError(
-                    (
-                        "config must be one of the strings 'default' or 'empty',",
-                        "or a FuzzyConfig object not,",
-                        f"{config} of type: {type(config)}.",
-                    )
-                )
-
-    def best_match(
-        self,
-        doc: Doc,
-        query: Doc,
-        fuzzy_func: str = "simple",
-        min_r1: int = 30,
-        min_r2: int = 75,
-        ignore_case: bool = True,
-        flex: Union[str, int] = "default",
-    ) -> Union[Tuple[int, int, int], None]:
-        """Returns the single best fuzzy match span in a Doc.
-
-        Finds the best fuzzy match span in doc based on the query,
-        assuming the minimum match ratios (min_r1 and min_r2) are met,
-        If more than one match has the same ratio,
-        the earliest match will be returned.
-
-        Args:
-            doc: Doc object to search over.
-            query: Doc object to fuzzy match against doc.
-            fuzzy_func: Key name of fuzzy matching function to use.
-                The default is "simple".
-                All fuzzywuzzy matching functions with default settings
-                are included:
-                "simple" = fuzz.ratio
-                "partial" = fuzz.partial_ratio
-                "token_set" = fuzz.token_set_ratio
-                "token_sort" = fuzz.token_sort_ratio
-                "partial_token_set" = fuzz.partial_token_set_ratio
-                "partial_token_sort" = fuzz.partial_token_sort_ratio
-                "quick" = fuzz.QRatio
-                "u_quick" = fuzz.UQRatio
-                "weighted" = fuzz.WRatio
-                "u_weighted" = fuzz.UWRatio
-            min_r1: Minimum fuzzy match ratio required for
-                selection during the intial search over doc.
-                This should be lower than min_r2 and "low" in general
-                because match span boundaries are not flexed initially.
-                0 means all spans of query length in doc will
-                have their boundaries flexed and will be recompared
-                during match optimization.
-                Lower min_r1 will result in more fine-grained matching
-                but will run slower. Default is 25.
-            min_r2: Minimum fuzzy match ratio required for
-                selection during match optimization.
-                Should be higher than min_r1 and "high" in general
-                to ensure only quality matches are returned.
-                Default is 75.
-            ignore_case: If strings should be lower-cased before
-                fuzzy matching or not. Default is True.
-            flex: Number of tokens to move match span boundaries
-                left and right during match optimization.
-                Default is "default".
-
-        Returns:
-            A tuple of the match span's start index,
-            end index, and fuzzy match ratio or None.
-
-        Raises:
-            TypeError: doc must be a Doc object.
-            TypeError: query must be a Doc object.
-
-        Example:
-            >>> import spacy
-            >>> from spaczz.fuzz import FuzzySearcher
-            >>> nlp = spacy.blank("en")
-            >>> searcher = FuzzySearcher()
-            >>> doc = nlp.make_doc("G-rant Anderson lives in TN.")
-            >>> query = nlp.make_doc("Grant Andersen")
-            >>> searcher.best_match(doc, query)
-            (0, 4, 90)
-        """
-        if not isinstance(doc, Doc):
-            raise TypeError("doc must be a Doc object.")
-        if not isinstance(query, Doc):
-            raise TypeError("query must be a Doc object.")
-        flex = self._calc_flex(query, flex)
-        match_values = self._scan_doc(doc, query, fuzzy_func, min_r1, ignore_case)
-        if match_values:
-            pos = self._index_max(match_values)
-            match = self._adjust_left_right_positions(
-                doc, query, match_values, pos, fuzzy_func, min_r2, ignore_case, flex,
-            )
-            return match
-        else:
-            return None
-
-    def match(
+    def compare(
         self, str1: str, str2: str, fuzzy_func: str = "simple", ignore_case: bool = True
     ) -> int:
         """Peforms fuzzy matching between two strings.
@@ -148,7 +63,7 @@ class FuzzySearcher:
             str1: First string for comparison.
             str2: Second string for comparison.
             fuzzy_func: Key name of fuzzy matching function to use.
-                All fuzzywuzzy matching functions with default settings
+                All rapidfuzz matching functions with default settings
                 are available:
                 "simple" = fuzz.ratio
                 "partial" = fuzz.partial_ratio
@@ -157,9 +72,8 @@ class FuzzySearcher:
                 "partial_token_set" = fuzz.partial_token_set_ratio
                 "partial_token_sort" = fuzz.partial_token_sort_ratio
                 "quick" = fuzz.QRatio
-                "u_quick" = fuzz.UQRatio
                 "weighted" = fuzz.WRatio
-                "u_weighted" = fuzz.UWRatio
+                "quick_lev" = fuzz.quick_lev_ratio
                 Default is "simple".
             ignore_case: Whether to lower-case str1 and str2
                 before comparison or not. Default is True.
@@ -170,15 +84,49 @@ class FuzzySearcher:
         Example:
             >>> from spaczz.fuzz import FuzzySearcher
             >>> searcher = FuzzySearcher()
-            >>> searcher.match("spaczz", "spacy")
+            >>> searcher.compare("spaczz", "spacy")
             73
         """
         if ignore_case:
             str1 = str1.lower()
             str2 = str2.lower()
-        return self._config.get_fuzzy_func(fuzzy_func, ignore_case)(str1, str2)
+        return round(self.get_fuzzy_func(fuzzy_func, ignore_case)(str1, str2))
 
-    def multi_match(
+    def get_fuzzy_func(
+        self, fuzzy_func: str, ignore_case: bool = True
+    ) -> Callable[[str, str], int]:
+        """Returns a fuzzy matching function based on it's key name.
+
+        Args:
+            fuzzy_func: Key name of the fuzzy matching function.
+            ignore_case: If fuzzy matching will be executed
+                with case sensitivity or not.
+
+        Returns:
+            A fuzzy matching function.
+
+        Raises:
+            ValueError: fuzzy_func was not a valid key name.
+
+        Example:
+            >>> from spaczz.fuzz import FuzzySearcher
+            >>> searcher = FuzzySearcher()
+            >>> simple = searcher.get_fuzzy_func("simple", False)
+            >>> simple("hi", "hi")
+            100.0
+        """
+        try:
+            return self._fuzzy_funcs[fuzzy_func]
+        except KeyError:
+            raise ValueError(
+                (
+                    f"No fuzzy matching function called {fuzzy_func}.",
+                    "Matcher must be in the following:",
+                    f"{list(self._fuzzy_funcs.keys())}",
+                )
+            )
+
+    def match(
         self,
         doc: Doc,
         query: Doc,
@@ -203,7 +151,7 @@ class FuzzySearcher:
                 If n is 0 all matches will be returned.
                 Defualt is 0.
             fuzzy_func: Key name of fuzzy matching function to use.
-                All fuzzywuzzy matching functions with default settings
+                All rapidfuzz matching functions with default settings
                 are available:
                 "simple" = fuzz.ratio
                 "partial" = fuzz.partial_ratio
@@ -212,9 +160,8 @@ class FuzzySearcher:
                 "partial_token_set" = fuzz.partial_token_set_ratio
                 "partial_token_sort" = fuzz.partial_token_sort_ratio
                 "quick" = fuzz.QRatio
-                "u_quick" = fuzz.UQRatio
                 "weighted" = fuzz.WRatio
-                "u_weighted" = fuzz.UWRatio
+                "quick_lev" = fuzz.quick_lev_ratio
                 Default is "simple".
             min_r1: Minimum fuzzy match ratio required for
                 selection during the intial search over doc.
@@ -253,7 +200,7 @@ class FuzzySearcher:
                 "chiken from Popeyes is better than chken from Chick-fil-A"
                 )
             >>> query = nlp.make_doc("chicken")
-            >>> searcher.multi_match(doc, query, ignore_case=False)
+            >>> searcher.match(doc, query, ignore_case=False)
             [(0, 1, 92), (6, 7, 83)]
         """
         if not isinstance(doc, Doc):
@@ -348,31 +295,31 @@ class FuzzySearcher:
         bmv_r = match_values[p_l]
         if flex:
             for f in range(1, flex + 1):
-                ll = self.match(
+                ll = self.compare(
                     query.text, doc[p_l - f : p_r].text, fuzzy_func, ignore_case
                 )
                 if (ll > bmv_l) and (p_l - f >= 0):
                     bmv_l = ll
                     bp_l = p_l - f
-                lr = self.match(
+                lr = self.compare(
                     query.text, doc[p_l + f : p_r].text, fuzzy_func, ignore_case
                 )
                 if (lr > bmv_l) and (p_l + f < p_r):
                     bmv_l = lr
                     bp_l = p_l + f
-                rl = self.match(
+                rl = self.compare(
                     query.text, doc[p_l : p_r - f].text, fuzzy_func, ignore_case
                 )
                 if (rl > bmv_r) and (p_r - f > p_l):
                     bmv_r = rl
                     bp_r = p_r - f
-                rr = self.match(
+                rr = self.compare(
                     query.text, doc[p_l : p_r + f].text, fuzzy_func, ignore_case
                 )
                 if rr > bmv_r and (p_r + f <= len(doc)):
                     bmv_r = rr
                     bp_r = p_r + f
-        r = self.match(query.text, doc[bp_l:bp_r].text, fuzzy_func, ignore_case)
+        r = self.compare(query.text, doc[bp_l:bp_r].text, fuzzy_func, ignore_case)
         if r >= min_r2:
             return (bp_l, bp_r, r)
         return None
@@ -423,7 +370,7 @@ class FuzzySearcher:
         match_values: Dict[int, int] = dict()
         i = 0
         while i + len(query) <= len(doc):
-            match = self.match(
+            match = self.compare(
                 query.text, doc[i : i + len(query)].text, fuzzy_func, ignore_case
             )
             if match >= min_r1:
@@ -440,7 +387,7 @@ class FuzzySearcher:
 
         By default flex is set to the legth of query.
         If flex is a value greater than query,
-        flex will be set to 1 instead.
+        flex will be set to len(query) instead.
 
         Args:
             query: The Doc object to fuzzy match with.
@@ -453,7 +400,7 @@ class FuzzySearcher:
             TypeError: If flex is not "default" or an int.
 
         Warnings:
-            UserWarning:
+            FlexWarning:
                 If flex is > len(query).
 
         Example:
@@ -540,25 +487,3 @@ class FuzzySearcher:
             return sorted(match_values, key=lambda x: (-match_values[x], x))[:n]
         else:
             return list(match_values.keys())
-
-    @staticmethod
-    def _index_max(match_values: Dict[int, int]) -> int:
-        """Returns the start index of the highest ratio fuzzy match or None.
-
-        If the max ratio applies to multiple indices
-        the lowest index will be returned.
-
-        Args:
-            match_values: Dict of potential fuzzy matches
-                in start index, fuzzy ratio pairs.
-
-        Returns:
-            Integer value of the best matches' start index.
-
-        Example:
-            >>> from spaczz.fuzz import FuzzySearcher
-            >>> searcher = FuzzySearcher()
-            >>> searcher._index_max({1:30, 9:100})
-            9
-        """
-        return sorted(match_values, key=lambda x: (-match_values[x], x))[0]
